package org.zeroturnaround.javarebel.integration.google;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtField;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Processes <code>com.google.appengine.tools.development.IsolatedAppClassLoader$ProxyPolicy</code>.
 * <p>
 * This prevents a ClassCircularityError
 * (com/google/appengine/tools/development/IsolatedAppClassLoader$ProxyPolicy$2).
 * 
 * @author Rein Raudjärv
 */
public class GaeProxyPolicyCBP extends JavassistClassBytecodeProcessor {
  
  private static final String FIELD_INITING = "_initializing"; 
  private static final String FIELD_INITED  = "_initialized"; 
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    
    ctClass.addField(CtField.make("private volatile boolean " + FIELD_INITING + " = false;", ctClass));
    ctClass.addField(CtField.make("private volatile boolean " + FIELD_INITED + " = false;", ctClass));
    
    try {
      CtMethod origMethod = ctClass.getDeclaredMethod("implies");
      CtMethod copyMethod = CtNewMethod.copy(origMethod, "_implies", ctClass, null);
      ctClass.addMethod(copyMethod);
      
      origMethod.setBody("{" +
            "  if (" + FIELD_INITING + ")" +
            "    return true;" +

            "  if (!" + FIELD_INITED + ")" +
            "    " + FIELD_INITING + " = true;" +
            
            "  try {" +
            "    boolean result = _implies($$);" +
            "    return result;" +
            "  } finally {" +
            "    " + FIELD_INITED + " = true;" +
            "    " + FIELD_INITING + " = false;" +
            "  }" +
            "}");
    } catch (NotFoundException e) {
    }
  }
}
