package org.zeroturnaround.javarebel.integration.resin;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtField;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.MethodCall;

/**
 * Processes <code>com.caucho.boot.ResinWatchdog</code>.
 * 
 * <p>
 * An extension to {@link WatchdogProcessCBP} only for 
 * <code>com.caucho.boot.ResinWatchdog</code> which is included in Resin 3.1.0 up to Resin 3.1.4.
 * </p>
 * <p>
 * This is a fix to a problem in which Watchdog and it's child process cannot establish a connection.
 * </p>
 * <p>Tested on Resin 3.1.3.</p>
 * 
 * @author Rein Raudjärv
 * 
 * @see WatchdogProcessCBP
 */
public class ResinWatchdogCBP extends WatchdogProcessCBP {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    super.process(cp, cl, ctClass);

    cp.importPackage("java.io");
    cp.importPackage("java.net");
    cp.importPackage("com.caucho.vfs");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.resin");

    final String processField = "__process";
    ctClass.addField(CtField.make("private java.lang.Process " + processField + ";", ctClass));

    final String jvmOutField = "__jvmOut";
    ctClass.addField(CtField.make("private com.caucho.vfs.WriteStream " + jvmOutField + ";", ctClass));

    ctClass.getDeclaredMethod("run").instrument(new ExprEditor() {
      public void edit(MethodCall m) throws CannotCompileException {
        if ("com.caucho.boot.ResinWatchdog".equals(m.getClassName())
            && "createProcess".equals(m.getMethodName())) {
          m.replace(
              "{" +
              "  Process process = $proceed($$);" +
              "  " + processField + " = process;" +
              "  " + jvmOutField + " = $5;" +
              "  $_ = process;" +
          "}");
        }

        else if ("java.net.ServerSocket".equals(m.getClassName())
            && "accept".equals(m.getMethodName())) {
          m.replace(
              "{" +
              "  Logger log = LoggerFactory.getInstance();" +

              "  $0.setSoTimeout(1000);" +
              "  InputStream stdIs = " + processField + ".getInputStream();" +
              "  OutputStream stdOs = " + processField + ".getOutputStream();" +
              "  WriteStream jvmOut = " + jvmOutField + ";" +
              "  byte[] data = new byte[1024];" +

              "  Socket s = null;" +
              "  for (int i = 0; i < 120 && s == null; i++) {" +
              "    if (log.isEnabled())" +
              "      log.log(\"JRebel-Resin: Listening for connections on port \" + $0.getLocalPort() + \" (\" + i + \")\");" +
              "    try {" +
              "      s = $0.accept();" +
              "    } catch (SocketTimeoutException e) {" +
              "    }" +

              "    while (stdIs.available() > 0) {" +
              "      int len = stdIs.read(data, 0, data.length);" +
              "      if (len < 0) {" +
              "        break;" +
              "      }" +
              "      jvmOut.write(data, 0, len);" +
              "      jvmOut.flush();" +
              "    }" +

              "  }" +

              "  if (log.isEnabled())" +
              "    log.log(\"JRebel-Resin: Accepted connection \" + s + \" on port \" + $0.getLocalPort());" +

              "  $_ = s;" +
          "}");
        }
      }
    });
  }

}
