package org.zeroturnaround.javarebel.integration.resin;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.javarebel.RebelServletContext;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>com.caucho.server.webapp.WebApp</code>
 * by registering Resin servlet context instance with JavaRebel.
 * <p>
 * The transformed class exists since Resin 3.1.
 * In Resin 3.0 the class was named as
 * <code>com.caucho.server.webapp.Application</code>.
 * The transformation code of that class is contained in the
 * <code>javarebel-servlet-plugin</code>.
 * 
 * @author Toomas Römer
 * @author Rein Raudjärv
 */
public class ResinWebAppCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    
    ctClass.addInterface(cp.get(RebelServletContext.class.getName()));
    
    ctClass.getMethod("init", "()V").insertAfter(
        "ServletIntegrationFactory.getInstance().registerServletContext((RebelServletContext) $0);");
    
    ctClass.getMethod("getRealPath", "(Ljava/lang/String;)Ljava/lang/String;").insertBefore(
        "{" + 
        "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
        "  RebelServletContext sc = (RebelServletContext) $0;" +
        "  if (integration.isResourceReplaced(sc, $1))" +
        "    return integration.getRealPath(sc, $1);" + 
        "}");
  }
}
